/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#include "data_structure.hpp"
#include "lib.rs.h"
#include "opendal.hpp"
#include "utils/ffi_converter.hpp"

namespace opendal {

void Lister::Destroy() noexcept {
  if (lister_) {
    ffi::delete_lister(lister_);
    lister_ = nullptr;
  }
}

Lister::Lister(ffi::Lister *lister) noexcept : lister_{lister} {}

Lister::Lister(Lister &&other) noexcept : lister_{other.lister_} {
  other.lister_ = nullptr;
}

Lister::~Lister() noexcept { Destroy(); }

std::optional<Entry> Lister::Next() {
  auto entry = lister_->next();

  if (!entry.has_value) {
    return std::nullopt;
  }

  return utils::parse_entry(std::move(entry.value));
}

}  // namespace opendal
